/************************************************
    RCU6093WB.c

  controlle for ROHM RCU6093W-B graphic display

    (c) 2022 Suwa-Koubou
*************************************************/

#include "HardwareProfile.h"
#include "pic_delay.h"
#include "RCU6093WB.h"

#include "debug.h"


/*-----------------------------------------------
    definition
-----------------------------------------------*/

#define iabs(x)   ((x)<0?-(x):(x))


/*-----------------------------------------------
    global variable
-----------------------------------------------*/

#include "font8x8.h"     //font8x8[96][8] 0x20- 0x7f
#include "font16x16.h"   //font16x16[][16] 0x2121-0x217e, 0x2221-0x227e, 0x2321-0x237e

static unsigned char vram[8][102];
static unsigned char bitptrn[8]={0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01,};

static int _xpos, _ypos;   // locate gvram position

/*-----------------------------------------------
    function prototype
-----------------------------------------------*/

static void gvram_write(unsigned char data);


/*===============================================
    lcd_init(void)
===============================================*/

void lcd_init(void)
{
    // RCU6093WB initialize
    lcd_SCLK = 1;
    lcd_DC_B = 1;
    lcd_SCEB = 1;
    lcd_RESET = 1;

    lcd_RESET = 0;
    delay_10us(1);
    lcd_RESET = 1;

    delay_1us();
  
    lcd_write(0x21, fCMD); // display=ON, X-ADDR increment, Extend instruction
    lcd_write(0xBC, fCMD); // display contrast
    lcd_write(0x14, fCMD); // LCD bias 1/7
    lcd_write(0x20, fCMD); // display=ON, X-ADDR increment, Basic instruction
    lcd_write(0x0C, fCMD); // display normal
    lcd_clear();
    lcd_write(X_ADDRESS_0SET, fCMD);
    lcd_write(Y_ADDRESS_0SET, fCMD);
    _xpos = _ypos = 0;

}


/*===============================================
    lcd_write(char data, int DC_flg)
===============================================*/

void lcd_write(char data, int DC_flg)
{
    int  n;

    lcd_DC_B = DC_flg;  // fDATA(1)/fCMD(0)

    lcd_SCLK = 0;
    lcd_SCEB = 0;

    for(n=0; n<8; n++){
        if(data & 0x80){
            lcd_SDIN = 1;
        }else{
            lcd_SDIN = 0;
        }
        data <<= 1;
        lcd_SCLK = 1;
        lcd_SCLK = 0;
    }

    lcd_SCEB = 1;
}


/*===============================================
    lcd_clear(void)
===============================================*/

void lcd_clear(void)
{
    int n;

    lcd_locate(0,0);
    for(n=0; n < 102*9; n++){
        lcd_write(0x00, fDATA);
    }
    lcd_locate(0,0);
}


/*===============================================
    lcd_all_white(void)
===============================================*/

void lcd_all_white(void)
{
    lcd_write(BASIC_INSTRUCTION, fCMD);
    lcd_write(DISPLAY_ALL_WHITE, fCMD);
}


/*===============================================
    lcd_all_black(void)
===============================================*/

void lcd_all_black(void)
{
    lcd_write(BASIC_INSTRUCTION, fCMD);
    lcd_write(DISPLAY_ALL_BLACK, fCMD);
}

/*===============================================
    lcd_locate(int row, int col)
      row:0-7     Y_ADDR = row
      col:0-11    X_ADDR = col*8+3 (3x2=6+96=102)
===============================================*/

void lcd_locate(int row, int col)
{
    lcd_write(X_ADDRESS_0SET+col*8+3, fCMD);
    lcd_write(Y_ADDRESS_0SET+row, fCMD);

    _xpos = col*8+3;
    _ypos = row;
}

/*===============================================
    lcd_putstr(char *str) // use font8x8[][]
===============================================*/

void lcd_putstr(int row, int col, unsigned char *str)
{
    unsigned int c;
    unsigned char data[8];
    int n;

    lcd_locate(row, col);

    while(*str){
      c = *str++;
      if(c < 0x20 || c > 0x7e) continue;
      convert_font8x8(data, &(font8x8[c-0x20][0]));
      for(n=0;n<8;n++){
          lcd_write(data[n], fDATA);
          gvram_write(data[n]);
      }
    }

}

void convert_font8x8(unsigned char data[], const unsigned char font[])
{
    int n, i;
    unsigned int mask, val;

    mask = 0x80;
    for(n=0; n<8; n++){
        data[n] = 0;
        for(val=0x80, i=0; i<8; i++, val>>=1){ 
            data[n] |= (font[i] & mask ?  val: 0);
       }
       mask >>= 1;
    }
}

/*===============================================
    lcd_putlstr(int row, int col, char *str) 
     use font8x8[][] double size to large font
===============================================*/

void lcd_putlstr(int row, int col, unsigned char *str)
{
    unsigned int c;
    unsigned short data[16];
    unsigned char *p;
    int n;

    p = str;
    lcd_locate(row-1,col);
    while(*p){
      c = *p++;
      if(c < 0x20 || c > 0x7e) continue;
      convert_largefont(data, &(font8x8[c-0x20][0]));
      for(n=0;n<16;n++){
          lcd_write((data[n]>>8)&0xff, fDATA);
          gvram_write((data[n]>>8)&0xff);
      }
    }

    p = str;
    lcd_locate(row,col);
    while(*p){
      c = *p++;
      if(c < 0x20 || c > 0x7e) continue;
      convert_largefont(data, &(font8x8[c-0x20][0]));
      for(n=0;n<16;n++){
          lcd_write(data[n]&0xff, fDATA);
          gvram_write(data[n]&0xff);
      }
    }

}

void convert_largefont(unsigned short data[], const unsigned char font[])
{
    int n, i;
    unsigned int mask, val;

    mask = 0x80;
    for(n=0; n<8; n++){
        data[n*2] = 0;
        data[n*2+1] = 0;
        for(val=0xc000, i=0; i<8; i++, val>>=2){ 
            data[n*2]   |= (font[i] & mask ?  val: 0);
            data[n*2+1] |= (font[i] & mask ?  val: 0);
        }
        mask >>= 1;
    }
}


/*===============================================
    lcd_putnum(char *str) // use font16x16[][]
      valid 13 char = +-.0123456789
===============================================*/

void lcd_putnum(int row, int col, unsigned char *str)
{
    unsigned int c;
    unsigned short data[16];
    const unsigned short *font;
    unsigned char *p;
    int n;

    p = str;
    lcd_locate(row-1,col);
    while(*p){
      c = *p++;
      if(c == '+'){
          font = &(font16x16[59][0]);
      }else if(c == '-'){
          font = &(font16x16[60][0]);
      }else if(c == '.'){
          font = &(font16x16[4][0]);
      }else if(c >= '0' && c <= '9'){
          font = &(font16x16[203+c-'0'][0]);
      }else{
          continue;
      }
      convert_font16x16(data, font);
      for(n=0;n<16;n++) {
          lcd_write((data[n]>>8)&0xff, fDATA);
          gvram_write((data[n]>>8)&0xff);
      }
    }

    p = str;
    lcd_locate(row,col);
    while(*p){
      c = *p++;
      if(c == '+'){
          font = &(font16x16[59][0]);
      }else if(c == '-'){
          font = &(font16x16[60][0]);
      }else if(c == '.'){
          font = &(font16x16[4][0]);
      }else if(c >= '0' && c <= '9'){
          font = &(font16x16[203+c-'0'][0]);
      }else{
          continue;
      }
      convert_font16x16(data, font);
      for(n=0;n<16;n++){
          lcd_write(data[n]&0xff, fDATA);
          gvram_write(data[n]&0xff);
      }
    }

}

void convert_font16x16(unsigned short data[], const unsigned short font[])
{
    int n, i;
    unsigned int mask, val;

    mask = 0x8000;
    for(n=0; n<16; n++){
        data[n] = 0;
        for(val=0x8000, i=0; i<16; i++, val>>=1){ 
            data[n] |= (font[i] & mask ?  val: 0);
       }
       mask >>= 1;
    }
}

/*===============================================
    glcd_pset(x,y,flg)
    x: 0-101
    y: 0-63
    flg: 1=set, 0=clear
===============================================*/

void glcd_pset(int x, int y, int flg)
{
    unsigned char *val;

    if(x<0 || x>101 || y<0 || y>63) return;

    val = &(vram[y/8][x]);
    if(flg){
        *val |= bitptrn[y%8];
    }else{
        *val &= ~(bitptrn[y%8]);
    }
}


/*===============================================
    glcd_clear()
    clear screen
===============================================*/

void glcd_clear(void)
{
    memset((void *)vram, 0, sizeof(vram));
}

/*===============================================
    glcd_line(int xBegin, int yBegin, int xEnd, int yEnd, int flg)
    xBegin,xEnd: 0-102
    yBegin,yEnd: 0-63
    flg: 1=set, 0=clear
===============================================*/


void glcd_line(int xBegin, int yBegin, int xEnd, int yEnd, int flg)
{
    int x, y, xlen, ylen;
    int temp, slant, offset;

    // range check
    if(xBegin < 0 || xEnd < 0 || yBegin < 0 || yEnd < 0) return; 
    if(xBegin > 101 || xEnd > 101 || yBegin > 63 || yEnd > 63) return;

    xlen=iabs(xEnd-xBegin);
    ylen=iabs(yEnd-yBegin);

    if(xlen>=ylen){
        if(xBegin > xEnd){
            temp=xBegin; xBegin=xEnd; xEnd=temp;
            temp=yBegin; yBegin=yEnd; yEnd=temp;
        }
        slant = ((yEnd-yBegin)*1024)/(xEnd-xBegin);
        offset = ((yBegin+yEnd)*1024-((((yEnd-yBegin)*(xBegin+xEnd))*1024)/(xEnd-xBegin)))/2;    

        for(x=xBegin; x<=xEnd; x++){
            y=slant*x+offset+512; y>>=10;
            glcd_pset(x, y, flg);
        }
    }else{
        if(yBegin > yEnd){
           temp=yBegin; yBegin=yEnd; yEnd=temp;
           temp=xBegin; xBegin=xEnd; xEnd=temp;
        }
        slant = ((xEnd-xBegin)*1024)/(yEnd-yBegin);
        offset = ((xBegin+xEnd)*1024-((((xEnd-xBegin)*(yBegin+yEnd))*1024)/(yEnd-yBegin)))/2;    

        for(y=yBegin; y<=yEnd; y++){
            x=slant*y+offset+512; x >>=10;
            glcd_pset(x, y, flg);
        }
    } 
}


/*===============================================
    glcd_screen(flg)
    flg: gON  display graphic vram
       : gOFF display off graphic vram 
===============================================*/

void glcd_screen(int flg)
{
    int x, y;

    lcd_write(X_ADDRESS_0SET, fCMD);
    lcd_write(Y_ADDRESS_0SET, fCMD);

    if(flg==gON){
        for(y=0; y < 8; y++){
            for(x=0; x<102; x++){
                lcd_write(vram[y][x], fDATA);
            }
        }
    }else{
        lcd_clear();
    }
}
 
/*===============================================
    gvram_write(data)
    write data to locate position on vram 
===============================================*/

static void gvram_write(unsigned char data)
{
    vram[_ypos][_xpos] = data;
    if(++_xpos > 101){
        _xpos=0;
        if(++_ypos > 7){
           _ypos=0;
        }
    }
}


/*** end of RCU6093W.c **************************/
