/******************************************************
    display.c

  display date&time, pressuer,
             temperature, humidity
   2022.2.2 (C) Suwa-Koubou
******************************************************/

#include <stdio.h>
#include <string.h>
#include "HardwareProfile.h"
#include "RCU6093WB.h"
#include "pic_delay.h"
#include "timer.h"
#include "ds3231.h"
#include "bme280.h"
#include "display.h"
#include "tskchg.h"

#include "debug.h"

/*------------------------------------------------------
    definition
------------------------------------------------------*/

#define LOC_yDATE    0
#define LOC_xDATE    2

#define LOC_yTIME    1
#define LOC_xTIME    4

#define LOC_yPRESS   3
#define LOC_xPRESS   2
#define LOC_xPUNIT   10

#define LOC_yTEMP    5
#define LOC_xTEMP    2
#define LOC_xTUNIT   10

#define LOC_yHUM     7
#define LOC_xHUM     2
#define LOC_xHUNIT   10


/*------------------------------------------------------
    global variable
------------------------------------------------------*/

// display data value
static unsigned char date[7];
static int press;
static int temperature;    // value * 10
static int humidity;       // value * 10

// clock display control
static int blink_item_num = -1;
static int tskDspClk = -1;

// display work buffer
static unsigned char pbuf[16];


/*------------------------------------------------------
    function prototype
------------------------------------------------------*/

/*======================================================
    set display data
======================================================*/

void set_date_value(unsigned char tm[])
{
    lock_timer();
    memcpy(date, tm, 7);
    unlock_timer();
}

void set_press_value(unsigned int press_val)
{
    lock_timer();
    press = press_val;
    unlock_timer();
}

void set_temp_value(int temp_val)
{
    lock_timer();
    temperature = temp_val;
    unlock_timer();
}

void set_humidity_value(int humidity_val)
{
    lock_timer();
    humidity = humidity_val;
    unlock_timer();
}

void clock_blink_display(int item_num, int blink_interval)
{
    blink_item_num = item_num;
    if(tskDspClk==-1){
        tskDspClk = tsk_entry(display_clock_set, 0, 0);  
    }
}

void clock_normal_display(void)
{
    if(tskDspClk != -1) tsk_delete(tskDspClk);
    tskDspClk = -1;
    blink_item_num = -1;
}


/*======================================================
    task display date time
======================================================*/

void display_datetime(void)
{
    for(;;){
        rtcReadNowTime(date);
        display_clock();
        tsk_delay(1000); // wait 1sec
    }
}

/*======================================================
    task display clock with blink
    timer callback by 1sec interval
======================================================*/

void display_clock_set(void)
{
    for(;;){
        display_clock();
        tsk_delay(300); // 300msec blink
    }
}

/*======================================================
    display clock
======================================================*/

void display_clock(void)
{
    unsigned char date_str[11];  // "YYYY/MM/DD\0"
    unsigned char time_str[9];   // "hh:mm:ss\0"
    static int item_pos[] = {6, 3, 0, 0, 8, 5, 2,};
    static int call_cnt;
    int pos;

    strcpy((char *)date_str, "2000/00/00"); date_str[10]=0;
    strcpy((char *)time_str, "00:00:00"); time_str[8]=0;

    date_str[2]+=((date[tm_YEAR]&0xf0)>>4);
    date_str[3]+= (date[tm_YEAR]&0x0f);

    date_str[5]+=((date[tm_MONTH]&0xf0)>>4);
    date_str[6]+= (date[tm_MONTH]&0x0f);

    date_str[8]+=((date[tm_DATE]&0xf0)>>4);
    date_str[9]+= (date[tm_DATE]&0x0f);

    time_str[0]+=((date[tm_HOUR]&0xf0)>>4);
    time_str[1]+= (date[tm_HOUR]&0x0f);

    time_str[3]+=((date[tm_MIN]&0xf0)>>4);
    time_str[4]+= (date[tm_MIN]&0x0f);

    time_str[6]+=((date[tm_SEC]&0xf0)>>4);
    time_str[7]+= (date[tm_SEC]&0x0f);

    // blinking ?
    if(blink_item_num != -1){
        if(call_cnt&1){
            // clear item part
            pos = item_pos[blink_item_num];
            if(blink_item_num >= tm_WDAY){
                date_str[pos] = ' ';
                date_str[pos+1] = ' ';
            }else{
                time_str[pos] = ' ';
                time_str[pos+1] = ' ';
            }
        }
        call_cnt++;
    }

    lcd_putstr(LOC_yDATE, LOC_xDATE, date_str);
    lcd_putstr(LOC_yTIME, LOC_xTIME, time_str);

}

/*======================================================
    task display enviroment information
    timer callback by 1minutes interval
======================================================*/

void debug_print_rtc_temp(void)
{
    unsigned char temp[2];

    rtcReadTemp(temp);

    putstr("rtcTemp="); putdec(temp[0]); putstr(".");
    putdec(((int)temp[1]*100)/256); putcrlf();
}

void display_env(void)
{
    for(;;){
        bme280ReadData(&press, &temperature, &humidity);
        display_press();
        display_temperature();
        display_humidity();
        debug_print_rtc_temp();
        tsk_delay(60000-10);    // wait 
    }
}


/*======================================================
    display pressuer
======================================================*/

void display_press(void)
{
    sprintf((char *)pbuf, "%4d", press);
    lcd_putlstr(LOC_yPRESS, LOC_xPRESS, pbuf);
    lcd_putstr(LOC_yPRESS, LOC_xPUNIT, (unsigned char *)"hp");
}


/*======================================================
    display temperature
======================================================*/

void display_temperature(void)
{

    if(temperature < 0){
        sprintf((char *)pbuf, "%2d.%d", temperature/10, - temperature%10);
    }else{
        sprintf((char *)pbuf, "%2d.%d", temperature/10, temperature%10);
    }

    lcd_putlstr(LOC_yTEMP, LOC_xTEMP, pbuf);
    lcd_putstr(LOC_yTEMP, LOC_xTUNIT, (unsigned char *)"`C");
}


/*======================================================
    display humidity
======================================================*/

void display_humidity(void)
{
    unsigned char pbuf[16];

    sprintf((char *)pbuf, "%2d.%d", humidity/10, humidity%10);
    lcd_putlstr(LOC_yHUM, LOC_xHUM, pbuf);
    lcd_putstr(LOC_yHUM, LOC_xHUNIT, (unsigned char *)"% ");
}

/*** end of display.c *********************************/
