/**************************************************************************
    ds3231.c

    Real Time Clock module, DS3231 for Pi

    2022.2.2  (c) Suwa-Koubou

***************************************************************************/

/** INCLUDES **************************************************************/
//#include <stdio.h>
//#include <stdlib.h>
#include "HardwareProfile.h"
#include "pic_delay.h"
#include "ds3231.h"


/** DEFINITION ************************************************************/
// Start/Stop
#define RTCSTART    1
#define RTCSTOP     0

// I2C data transfer mode bit
#define MW          0   // I2C master write mode
#define MR          1   // I2C master read mode

#define I2C_HANDLING    // use i2c access routines in this file

/*==========================================================================
    Real Time Clock module, DS3231
==========================================================================*/

/*--------------------------------------------------------------------------
    RTC initialize
--------------------------------------------------------------------------*/

void    rtcInit(void)
{
    delay_ms(100);

    i2cOpenStart();
    i2cSend(RTCDEVADR + MW);
    i2cSend(RTCCONTRL);
    i2cSend(0x00);  // start
    i2cSend(0x00);  // clear OSF bit
    i2cStopClose();

    delay_us(5);

}


/*--------------------------------------------------------------------------
    RTC Write Time
    tm[0] = sec, tm[1] = min, tm[2] = hour,
    tm[3] = wday, tm[4] = date, tm[5] = month, tm[6] = year;    
--------------------------------------------------------------------------*/

void    rtcWriteTime(unsigned char tm[])
{
    short   i;

    // masked register invalid bits
    tm[0] &= 0x7f;  // second
    tm[1] &= 0x7f;  // minute
    tm[2] &= 0x3f;  // set 24 hour 
    tm[3] &= 0x07;  // wday
    tm[4] &= 0x3f;  // date
    tm[5] &= 0x1f;  // month, clear century bit


    // set Date & Time
    i2cOpenStart();
    i2cSend(RTCDEVADR + MW);
    i2cSend(RTCSEC);
    for(i = 0; i < 7; i++){
        i2cSend(tm[i]);
    }
    i2cStopClose();

}   


/*--------------------------------------------------------------------------
    RTC Read Time
    tm[0] = sec, tm[1] = min, tm[2] = hour,
    tm[3] = wday, tm[4] = date, tm[5] = month, tm[6] = year;    
--------------------------------------------------------------------------*/

void    rtcReadTime(unsigned char tm[])
{
    short   i;

    i2cOpenStart();
    i2cSend(RTCDEVADR + MW);
    i2cSend(RTCSEC);
    // i2cReStart();
    i2cStopClose();
    i2cOpenStart();
    i2cSend(RTCDEVADR + MR);
    for(i = 0; i < 7; i++){
        tm[i] = i2cRec();
        if(i < 6){
            i2cAckSend();
        }else{
            i2cNAckSend();
        }
    }
    i2cStopClose();

    // masked register invalid bits
    tm[0] &= 0x7f;  // second
    tm[1] &= 0x7f;  // minute
    tm[2] &= 0x3f;  // hour
    tm[3] &= 0x07;  // wday
    tm[4] &= 0x3f;  // date
    tm[5] &= 0x1f;  // month
}   


void    rtcReadNowTime(unsigned char tm[])
{
    unsigned char   data[2][7];
    int i, prev;

    // first read
    rtcReadTime(&(data[0][0]));
    prev = 1;
    for(;;){
        // next read
        rtcReadTime(&(data[prev][0]));
        for(i = 0; i < 7; i++){
            if(data[0][i] != data[1][i]) break;
        }
        if(i >= 7) break;   // match break
        prev ^= 1;  // read retry
    }   

    for(i = 0; i < 7; i++){
        tm[i] = data[0][i];
    }
}

/*--------------------------------------------------------------------------
    RTC Read Temp
--------------------------------------------------------------------------*/

void rtcReadTemp(unsigned char temp[])
{
    i2cOpenStart();
    i2cSend(RTCDEVADR + MW);
    i2cSend(RTCTEMPHIGH);
    // i2cReStart();
    i2cStopClose();
    i2cOpenStart();
    i2cSend(RTCDEVADR + MR);
    temp[0] = i2cRec();
    i2cAckSend();
    temp[1] = i2cRec();
    i2cNAckSend();
    i2cStopClose();

}

/*--------------------------------------------------------------------------
    RTC Read Status
--------------------------------------------------------------------------*/

void rtcReadStatus(unsigned char *status)
{
    i2cOpenStart();
    i2cSend(RTCDEVADR + MW);
    i2cSend(RTCSTATUS);
    // i2cReStart();
    i2cStopClose();
    i2cOpenStart();
    i2cSend(RTCDEVADR + MR);
    *status = i2cRec();
    i2cNAckSend();
    i2cStopClose();
}



#ifdef  I2C_HANDLING

/*==========================================================================
    I2C Handling module
==========================================================================*/
#define TIME_OUT    0xffff
static unsigned short tw;

#define  BRG_40M_400KHZ     0x02c
#define  BRG_40M_100KHZ     0x0c2

/*--------------------------------------------------------------------------
    I2C master Open and Start
--------------------------------------------------------------------------*/

void    i2cOpenStart(void)
{
    I2C1BRG = BRG_40M_400KHZ;
    //I2C1BRG = BRG_40M_100KHZ;

    // enabled I2C
    I2C1CONbits.I2CEN = 1;

    I2C1CONbits.I2CEN = 1;
    delay_us(5);

    // START 
    I2C1CONbits.SEN = 1;
    for(tw = TIME_OUT; tw > 0; tw--){
        if(!I2C1CONbits.SEN) break;
    }
}


/*--------------------------------------------------------------------------
    I2C master ReStart
--------------------------------------------------------------------------*/

void    i2cReStart(void)
{
    // RESTART 
    I2C1CONbits.RSEN = 1;
    for(tw = TIME_OUT; tw > 0; tw--){
        if(!I2C1CONbits.RSEN) break;
    }
}


/*--------------------------------------------------------------------------
    I2C master Stop and Close
--------------------------------------------------------------------------*/

void    i2cStopClose(void)
{
    // STOP 
    I2C1CONbits.PEN = 1;
    for(tw = TIME_OUT; tw > 0; tw--){
        if(!I2C1CONbits.PEN)break;
    }
    delay_us(5);

    // disabled I2C
    I2C1CONbits.I2CEN = 0;
}


/*--------------------------------------------------------------------------
    I2C master data send
--------------------------------------------------------------------------*/

int i2cSend(unsigned char data)
{
    I2C1TRN = data;

    if(I2C1STATbits.IWCOL){ // If write collision occurs,return -2
        return -2;
    }

    for(tw = TIME_OUT; tw > 0; tw--){
        if(!I2C1STATbits.TRSTAT) break;  // wait until write cycle is complete 
    }

    if (I2C1STATbits.ACKSTAT){  // test for ACK condition received
        return -1;  // NACK received
    }

    return 0;   // complete with no error.
}


/*--------------------------------------------------------------------------
    I2C master data receive
--------------------------------------------------------------------------*/

unsigned char   i2cRec(void)
{
    // enabled Receive
    I2C1CONbits.RCEN = 1;

    // wait until receive complete and data stored to I2CRCV register
    for(tw = TIME_OUT; tw > 0; tw--){
        if(!I2C1CONbits.RCEN && I2C1STATbits.RBF) break;
    }

    // clear overflow bit
    I2C1STATbits.I2COV = 0;
    
    return (unsigned char)I2C1RCV;
}


/*--------------------------------------------------------------------------
    I2C master Ack send
--------------------------------------------------------------------------*/

void    i2cAckSend(void)
{
    // Ack Send
    I2C1CONbits.ACKDT = 0;
    I2C1CONbits.ACKEN = 1;

    // wait until ack cycle complete
    for(tw = TIME_OUT; tw > 0; tw--){
        if(!I2C1CONbits.ACKEN)break;
    }
}


/*--------------------------------------------------------------------------
    I2C master NotAck send
--------------------------------------------------------------------------*/

void    i2cNAckSend(void)
{
    // NAck Send
    I2C1CONbits.ACKDT = 1;
    I2C1CONbits.ACKEN = 1;

    // wait until ack cycle complete
    for(tw = TIME_OUT; tw > 0; tw--){
        if(!I2C1CONbits.ACKEN) break;
    }
}


#endif  // I2C_HANDLING

/*** end of ds3231.c *****************************************************/
