/*****************************************************************************

    main.c   for PIC32MX230F064B

    ROHM RCU6093W-B 102x65 Graphick LCD
    DS3231 real time clock
    BME280 environment monitoring sensor


    pin assign PIC32MX230F064B
                     MCLR 1    28 AVDD
    debug UART1/TX -- RA0 2    27 AVSS
    LM61BIZ temp. --- RA1 3    26 RB15   -- DOWN button
    RCU6093W SDIN --- RB0 4    25 RB14   -- UP button
    RCU6093W DC_B --- RB1 5    24 RB13   -- SET button
    RCU6093W SCEB --- RB2 6    23 Vusb3v3
    RCU6093W SCLK --- RB3 7    22 RB11/PGEC2
                  GND/VSS 8    21 RB10/PGED2
  CeraRock20 -+- OSC1/RA2 9    20 VCAP via 10uF to GND
              +- OSC2/RA3 10   19 GND
                      RB4 11   18 RB9/SDA1  --- BME280-SDA DS3231-SDA
    debug UART1/RX -- RA4 12   17 RB8/SCL1  --- BME280-SCL DS3231-SCL
                      VDD 13   16 RB7   
    RCU6093W RESET -- RB5 14   15 VBUS


//  2022.4.11  xZT[BME280LM61BIZɕύX

     interrupt level
       1: 1msec timer
       2: UART1


*****************************************************************************/

#include "HardwareProfile.h"
#include "RCU6093WB.h"
#include "ds3231.h"
#include "pic_delay.h"
#include "key.h"
#include "timer.h"
#include "display.h"
#include "bme280.h"
#include "tskchg.h"

#include "debug.h"

#include "p32mx230f_frc_config.h"

/*----------------------------------------------------------------------------
    definition
----------------------------------------------------------------------------*/

#define INFO_MAX           100
#define ENVINFO_STORE_TIME 1800000L   // env info store interval time. 30min

#define PRESS_MINIMUM       970
#define PRESS_MAXIMUM      1100

#define TEMP_MINIMUM       -50    // tempx10
#define TEMP_MAXIMUM       500    // tempx10

#define HUM_MINIMUM        200    // humx10
#define HUM_MAXIMUM        700    // humx10


/*----------------------------------------------------------------------------
    function prototype
----------------------------------------------------------------------------*/

int  main(void);
void exec_loop(void);
void clock_set(void);
void env_draw_line(void);
void draw_line_graph(int sel, int min, int max, short data[]);
void storeEnvInfo(void);
void envInfoWrite(unsigned char tm[], unsigned short press, short temp, unsigned short hum);
void envInfoRead(unsigned short press[], short temp[], unsigned short hum[]);
void io_initialize(void);


/*----------------------------------------------------------------------------
    global variable
----------------------------------------------------------------------------*/

typedef struct {
    unsigned char date[6]; // year, month, day, wday, hour, min
    unsigned short press;
    short          temp;
    unsigned short hum;
} ENVINFO;

static ENVINFO envInfo[INFO_MAX];
static int envWritePtr=0;
static int envReadPtr=0;
static int envSize=0;

static unsigned short    press_data[INFO_MAX];
static short       temperature_data[INFO_MAX];
static unsigned short humidity_data[INFO_MAX];


/*============================================================================
    main(void)
============================================================================*/

int main(void)
{
    io_initialize();

    UART1Init();
    putstr("*** Start Environmental Monitoring System. \r\n");
    putstr("sizeof(ENVINFO)="); putdec(sizeof(ENVINFO)); putcrlf();

    rtcInit();
    lcd_init();
    bme280Init();
    timer_init();
    tsk_init();

    exec_loop();

    return 0;
}


/*============================================================================
    exec_loop()
============================================================================*/

void exec_loop(void)
{
    unsigned char tm[7];
    int tskDspEnv;
    int tskDspDate;
    int tskStrEnv;
    int key;

    rtcReadNowTime(tm);
    set_date_value(tm);
    display_clock();

    tskDspDate = tsk_entry(display_datetime, 0, 0);
    tskDspEnv = tsk_entry(display_env, 0, 0);
    tskStrEnv = tsk_entry(storeEnvInfo, 0, 0);

    for(;;){
        while((key=getkey())==KEY_NONE) tsk_chg(0);

        // clock set
        if(key==KEY_SET){
            tsk_delete(tskDspDate);
            clock_set();
            tskDspDate = tsk_entry(display_datetime, 0, 0);

        }else if(key==KEY_UP){
            tsk_delete(tskDspEnv);
            env_draw_line();
            lcd_clear();
            tskDspEnv = tsk_entry(display_env, 0, 0);
        }
    }
}


/*======================================================
    Clock Set Operation
======================================================*/

#define  bcd2bin(x)     ((((x)>>4)&0xf)*10+((x)&0xf))
#define  bin2bcd(x)     ((((x)/10)<<4)+(x)%10)

void clock_set(void)
{                       // sec min hour wday date month year
    static int max[7] = { 59, 59, 23,  7,   31,  12,   99, };
    static int min[7] = {  0,  0,  0,  1,   1,   1,    0, };
    unsigned char tm[7];
    int key, i, val;

    // read Date & Time
    rtcReadNowTime(tm);
    set_date_value(tm);

    // Set from YYYY to ss (YYYY/MM/DD hh:mm:ss)
    // display control change
    for(i=6; i >= 0; i--){
        if(i == tm_WDAY) continue;

        // display control change
        clock_blink_display(i, 300);  // 300msec

        for(;;){    

        // display clock
            set_date_value(tm);

        // wait for key 
        while((key = getkey()) == KEY_NONE) tsk_chg(0);

            val = bcd2bin(tm[i]);

            if(key == KEY_UP){
                if(++val > max[i]) val = min[i];
            }else if(key == KEY_DOWN){
                if(--val < min[i]) val = max[i];
            }else if(key == KEY_SET){
                break;   // to next tm[i]
            }else{
                // reset & display
            rtcReadNowTime(tm);
                set_date_value(tm);
                clock_normal_display();
                display_clock();
                return;  // cancel 
            }
            tm[i] = bin2bcd(val);
        }
    }

    // set Now Date & Time to RTCC
    rtcWriteTime(tm);

    // display control change 
    clock_normal_display();
}



/*======================================================
    store environment information data task
======================================================*/

void storeEnvInfo(void)
{
    unsigned char tm[7];
    int press, temp, hum;

    for(;;){
        // Ɋi[
        rtcReadNowTime(tm);
        bme280ReadData(&press, &temp, &hum);
        envInfoWrite(tm, (unsigned short)press, (short)temp, (unsigned short)hum);
        putstr("store envInfo. size=");putdec(envSize);putstr(",time=");puthex(tm[tm_MIN]); putcrlf();

        tsk_delay(ENVINFO_STORE_TIME);  // store interval time .
    }

}

/*======================================================
    environ data write
======================================================*/

void envInfoWrite(unsigned char tm[], unsigned short press, short temp, unsigned short hum)
{
    ENVINFO *p;

    p=&(envInfo[envWritePtr]);
    memcpy(p->date, tm, 6);
    p->press = press;
    p->temp = temp;
    p->hum = hum;

    envWritePtr++;
    envWritePtr %= INFO_MAX;

    if(++envSize >= INFO_MAX){
        envSize=INFO_MAX;
        envReadPtr=envWritePtr+1;
        envReadPtr%=INFO_MAX;
    }
}

/*======================================================
    environ data read
======================================================*/

void envInfoRead(unsigned short press[], short temp[], unsigned short hum[])
{
    int n, rp;

    for(rp=envReadPtr, n=0; n<envSize; n++){
        press[n]=envInfo[n].press;
        temp[n]=envInfo[n].temp;
        hum[n]=envInfo[n].hum;
        rp++;
        rp%=INFO_MAX;
    }
}


/*======================================================
    environ data draw graphic line
======================================================*/

void env_draw_line(void)
{
    int key;
    int sel;

    envInfoRead(press_data, temperature_data, humidity_data);

    for(sel=0;;sel++, sel%=3){
        if(sel == 0){
            draw_line_graph(sel, PRESS_MINIMUM, PRESS_MAXIMUM, (short *)press_data);
        }else if(sel == 1){
            draw_line_graph(sel, TEMP_MINIMUM, TEMP_MAXIMUM, (short *)temperature_data);
        }else if(sel == 2){
            draw_line_graph(sel, HUM_MINIMUM, HUM_MAXIMUM, (short *)humidity_data);
        }
        while((key=getkey())!=KEY_UP && key!=KEY_SET) tsk_chg(0);
        if(key == KEY_SET) break;
    }

}
            

void draw_line_graph(int sel, int min, int max, short data[])
{
    int  n,x,y;
    int  scale;
    int  max_val, min_val;
    unsigned char pbuf[16];

    glcd_clear();

    max_val = 0x80000000;
    min_val = 0x7fffffff;
    for(n=0;n<envSize;n++){
        if((int)data[n] < min_val){
            min_val = data[n];
        }
        if((int)data[n] > max_val){
            max_val = data[n];
        }
    }
    // display max, min value
    if(sel==0){ // press
        lcd_putstr(2, 4, (unsigned char *)"PRESS");
        sprintf((char *)pbuf, "max:%4d", max_val);
        lcd_putstr(3, 4, pbuf);
        sprintf((char *)pbuf, "min:%4d", min_val);
        lcd_putstr(4, 4, pbuf);

    }else if(sel==1){
        lcd_putstr(2, 4, (unsigned char *)"TEMP");
        if(max_val < 0){
            sprintf((char *)pbuf, "max:%2d.%d", max_val/10, - max_val%10);
        }else{
            sprintf((char *)pbuf, "max:%2d.%d", max_val/10, max_val%10);
        }
        lcd_putstr(3, 4, pbuf);
        if(max_val < 0){
            sprintf((char *)pbuf, "min:%2d.%d", min_val/10, - min_val%10);
        }else{
            sprintf((char *)pbuf, "min:%2d.%d", min_val/10, min_val%10);
        }
        lcd_putstr(4, 4, pbuf);

    }else if(sel==2){
        lcd_putstr(2, 4, (unsigned char *)"HUMI");
        sprintf((char *)pbuf, "max:%2d.%d", max_val/10, max_val%10);
        lcd_putstr(6, 4, pbuf);
        sprintf((char *)pbuf, "min:%2d.%d", min_val/10, min_val%10);
        lcd_putstr(7, 4, pbuf);
    }

    // decided scaling factor
    scale = 36*1024/(max-min);  // dot60 - dot24

    // draw line graph
    for(n=0,x=1;n<envSize;n++,x++){  // 100data
        y= 60 - ((scale * ((int)data[n]-min)+512)>>10);
        glcd_pset(x, y, gON);
    }


    glcd_line(0, 16, 0, 63, 1);
    glcd_line(0, 63, 101, 63, 1);

    glcd_screen(gON);
}


/*===============================================================================
    read Temperature from LM61BIZ
===============================================================================*/

int readTempFromLM61BIZ(void)
{
    int temp;

    AD1CHSbits.CH0SA = 1; // using RA1(AN1)
    AD1CON1bits.DONE = 0; // clear previous done bit
    AD1CON1bits.SAMP = 1; // start sampling
    while(! AD1CON1bits.DONE);  // wait for conversion done
    temp = ADC1BUF0;

    temp = (temp * 3331)/1023 -600;   // temp*10  25.1 -> 251
    return temp;
}


/*----------------------------------------------------------------------------
    io_initiailize(void)
----------------------------------------------------------------------------*/

void io_initialize(void)
{
    unsigned int i;

    i = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );
    CheKseg0CacheOn();
    INTEnableSystemMultiVectoredInt();
    i = OSCCON;
    while (!(i & 0x00000020)){
        i = OSCCON;    // Wait for PLL lock to stabilize
    }
    INTEnableInterrupts();

    // disable JTAG
    DDPCONbits.JTAGEN = 0;


    //--- setting I/O port ----------------------------------------
    ANSELA=0x0002;  // RA1 is analog input
    ANSELB=0;

    LATA = 0x0000; TRISA = 0xffff;              // RA are default input.
    LATB = 0x003F; TRISB = 0b1111111111000000;  // RB5-RB0 are output

    RPA0R = 0b0001;       // RA0 is assigned to U1TX
    LATAbits.LATA0 = 1;   // RA0(U1TX) to level high
    TRISAbits.TRISA0 = 0; // RA0(U1TX) is output
    U1RXR = 0b0010;       // RA4 is assined to U1RX
    TRISAbits.TRISA4 = 1; // RA4(U1RX) is input

    CNPUBbits.CNPUB13 = 1;  // SET key input
    CNPUBbits.CNPUB14 = 1;  // UP key input
    CNPUBbits.CNPUB15 = 1;  // DOWN key input

    I2C1CONbits.DISSLW = 1; // shuld be set when RA0,RA1 is used with using I2C1.


    // ADC initialize
    AD1CON1bits.ADON = 0;   // AD converter is off
    AD1CON1bits.ADSIDL = 0; // Continue module operation in idle mode
    AD1CON1bits.FORM = 0;   // integer (DOUT= 0000 dddd dddd dddd)
    AD1CON1bits.SSRC = 0b111; // internal counter ends sampling and starts conversion(auto convert)
    AD1CON1bits.ASAM = 0;   // Sampling begins when SAMP bit set

    AD1CON2bits.VCFG = 0;   // reference voltage is AVdd, AVss
    AD1CON2bits.CSCNA = 0;  // Do not scan inputs 
    //ADCON2bits.CSCNA = 1; // Scan inputs 
    AD1CON2bits.SMPI = 0;   // interrupt at the cpmpletion of conversion for each sample/convert sequence
    AD1CON2bits.BUFM = 0;   // Buffer configured as one 16-word buffer ADCBUF(15...0)
    AD1CON2bits.ALTS = 0;   // Always use MUX A input multiplexer setting.

    AD1CON3bits.ADRC = 0;   // Clock derived from system clock

    AD1CON3bits.SAMC = 13;  // Auto sample time is 13 * Tad
    AD1CON3bits.ADCS = 19;  // AD conversion clock select, Tad = 20Tcy
    // Tcy=40Mhz 0.025usec
    // Tad=20Tcy=20*0.025usec=0.5usec
    // (12+13)Tad=25*0.5=12.5usec

    AD1CHSbits.CH0NA = 0;   // Negative input Vref-
    AD1CHSbits.CH0SA = 0;   // Positive input AN0
    AD1CSSL = 0x0000;               // skip all channel input scan

    AD1CON1bits.ADON = 1;    // turn ADC ON
}

/*** end of main.c **********************************************************************/
